@@ -42,7 +42,7 @@ def safe_collate(batch):
 def num_workers(args, loader_args, distributed=False):
     if distributed:
         return (loader_args['num_workers'] + args['num_gpus'] - 1) // args['num_gpus']
-    return loader_args['num_workers']
+    return loader_args.get('num_workers', 0)
 
 
 def batch_size(args, loader_args, distributed=False):
@@ -71,18 +71,17 @@ def _et_data_job(mode, arg, dspec, cls, total, func, verbose, i, file):
 class ETDataHandle:
 
     def __init__(self, args=None, dataloader_args=None, **kw):
-        self.dataset = {}
-        self.dataloader = {}
         self.args = _etutils.FrozenDict(args)
         self.dataloader_args = _etutils.FrozenDict(dataloader_args)
+        self.datasets = {}
 
-    def get_dataset(self, handle_key, files, dataspec: dict, reuse=False, dataset_cls=None):
-        if reuse and self.dataset.get(handle_key):
-            return self.dataset[handle_key]
-
+    def get_dataset(self, handle_key, files, dataspec: dict, reuse=True, dataset_cls=None):
+        if reuse and self.datasets.get(handle_key):
+            return self.datasets[handle_key]
         dataset = dataset_cls(mode=handle_key, limit=self.args['load_limit'], **self.args)
         dataset.add(files=files, verbose=self.args['verbose'], **dataspec)
-        self.dataset[handle_key] = dataset
+        if reuse:
+            self.datasets[handle_key] = dataset
         return dataset
 
     def get_train_dataset(self, split_file, dataspec: dict, dataset_cls=None):
@@ -123,21 +122,16 @@ class ETDataHandle:
             if len(datasets) > 0 and sum([len(t) for t in datasets if t]) > 0:
                 return datasets
 
-    def get_loader(self,
-                   handle_key='', distributed=False,
-                   use_unpadded_sampler=False,
-                   reuse=False, **kw
-                   ):
-
-        if reuse and self.dataloader.get(handle_key) is not None:
-            return self.dataloader[handle_key]
-
+    def get_loader(self, handle_key='', distributed=False, use_unpadded_sampler=False, **kw):
         args = {**self.args}
         args['distributed'] = distributed
         args['use_unpadded_sampler'] = use_unpadded_sampler
         args.update(self.dataloader_args.get(handle_key, {}))
         args.update(**kw)
 
+        if args.get('dataset') is None:
+            return None
+
         loader_args = {
             'dataset': None,
             'batch_size': 1,
@@ -150,6 +144,7 @@ class ETDataHandle:
             'timeout': 0,
             'worker_init_fn': _seed_worker if args.get('seed_all') else None
         }
+
         for k in loader_args.keys():
             loader_args[k] = args.get(k, loader_args.get(k))
 
@@ -172,8 +167,7 @@ class ETDataHandle:
             loader_args['num_workers'] = num_workers(args, loader_args, True)
             loader_args['batch_size'] = batch_size(args, loader_args, True)
 
-        self.dataloader[handle_key] = _DataLoader(collate_fn=safe_collate, **loader_args)
-        return self.dataloader[handle_key]
+        return _DataLoader(collate_fn=safe_collate, **loader_args)
 
     def create_splits(self, dataspec, out_dir):
         if _du.should_create_splits_(out_dir, dataspec, self.args):
@@ -202,13 +196,14 @@ class ETDataHandle:
         for ix, f in enumerate(files, 1):
             _files.append([ix, f])
 
-        nw = min(num_workers(args, args, args['use_ddp']), len(_files))
+        nw = min(num_workers(args, args, args.get('use_ddp')), len(_files))
         with _mp.Pool(processes=max(1, nw)) as pool:
-            return list(
+            dataset_list = list(
                 pool.starmap(
-                    _partial(_et_data_job, mode, args, dataspec, dataset_cls, len(_files), func, args['verbose']),
+                    _partial(_et_data_job, mode, args, dataspec, dataset_cls, len(_files), func, args.get('verbose')),
                     _files)
             )
+            return [_d for _d in dataset_list if len(_d) >= 1]
 
     @staticmethod
     def pooled_load(split_key, dataspecs, args, dataset_cls, load_sparse=False,
@@ -366,8 +361,11 @@ class UnPaddedDDPSampler(_data.Sampler):
         self.num_replicas = num_replicas
         self.rank = rank
         self.epoch = 0
-        self.num_samples = int(_math.ceil(len(self.dataset) * 1.0 / self.num_replicas))
-        self.total_size = self.num_samples * self.num_replicas
+
+        """For unpadded sampling"""
+        self.num_samples = int(_math.ceil((len(self.dataset) - self.rank) * 1.0 / self.num_replicas))
+        self.total_size = len(self.dataset)
+
         self.shuffle = shuffle
         self.seed = seed
 
