@@ -265,18 +265,17 @@ class EasyTorch:
         if best_exists and (self.args['phase'] == Phase.TRAIN or self.args['pretrained_path'] is None):
             """ Best model will be split_name.pt in training phase, and if no pretrained path is supplied. """
             trainer.load_checkpoint(trainer.cache['log_dir'] + _sep + trainer.cache['best_checkpoint'],
-                                    map_location=trainer.device['gpu'])
+                                    map_location=trainer.device['gpu'], load_optimizer_state=False)
 
         """ Run and save experiment test scores """
-        if test_dataset is not None:
-            test_out = trainer.evaluation(mode='test', save_pred=True, distributed=False, dataset=test_dataset)
-            test_scores = trainer.reduce_scores([test_out], distributed=False)
-            trainer.cache[LogKey.TEST_METRICS] = [[split_file,
-                                                   *test_scores['averages'].get(),
-                                                   *test_scores['metrics'].get()]]
-            _utils.save_scores(trainer.cache, experiment_id=trainer.cache['experiment_id'],
-                               file_keys=[LogKey.TEST_METRICS])
-            return test_out
+        test_out = trainer.inference(mode='test', save_predictions=True, datasets=test_dataset)
+        test_scores = trainer.reduce_scores([test_out], distributed=False)
+        trainer.cache[LogKey.TEST_METRICS] = [[split_file,
+                                               *test_scores['averages'].get(),
+                                               *test_scores['metrics'].get()]]
+        _utils.save_scores(trainer.cache, experiment_id=trainer.cache['experiment_id'],
+                           file_keys=[LogKey.TEST_METRICS])
+        return test_out
 
     def run(self, trainer_cls: typing.Type[ETTrainer],
             dataset_cls: typing.Type[ETDataset] = None,
@@ -324,10 +323,9 @@ class EasyTorch:
                     self._train(trainer, train_dataset, validation_dataset, dspec)
 
                 if self.args['is_master']:
-                    test_dataset = trainer.data_handle.get_test_dataset(
-                        split_file, dspec, dataset_cls=dataset_cls
-                    )
-                    test_accum.append(self._test(split_file, trainer, test_dataset))
+                    test_dataset = trainer.data_handle.get_test_dataset(split_file, dspec, dataset_cls=dataset_cls)
+                    if test_dataset is not None:
+                        test_accum.append(self._test(split_file, trainer, test_dataset))
 
                 if trainer.args.get('use_ddp'):
                     _dist.barrier()
@@ -383,7 +381,7 @@ class EasyTorch:
             train_dataset = ETDataHandle.pooled_load('train', self.dataspecs, self.args,
                                                      dataset_cls=dataset_cls, load_sparse=False)[0]
             val_dataset = ETDataHandle.pooled_load('validation', self.dataspecs, self.args,
-                                                   dataset_cls=dataset_cls, load_sparse=False)
+                                                   dataset_cls=dataset_cls, load_sparse=False)[0]
             self._train(trainer, train_dataset, val_dataset, {'dataspecs': self.dataspecs})
 
         """Only do test in master rank node"""
